#ifndef WAVELET_OPENCL_H
#define WAVELET_OPENCL_H

#include <string>
#include <climits>
#include <cmath>
#include <omp.h>

#include "Debug.h"
#include "main.h"
#include "Wavelet.h"
#include "WaveletOpenclParam.h"

/*
 * @brief Wald boost detector filter class.
 */

typedef std::vector<cl_event> v_cl_event;

class WaveletOpencl : public Wavelet
{
	public:
    /**
     * Destructor to delete allocated data.
     */
    virtual bool isValid();
    /**
    * Transcode frame.
    * @param bgra_frame frame in BGRA format to transcode
    * @param out optional frame process timers
    * @param error optional string to write error
    */
    virtual bool getFrame(uchar4 *bgra_frame, det_output *out = NULL) = 0;
  protected:
    WaveletOpencl(rect image_size, WaveletOpenclParam *param);
    ~WaveletOpencl();
    bool getFramePre(uchar4 *bgra_frame);
    bool getFramePost(uchar4 *bgra_frame, cl_mem out_image, cl_mem in_image, det_output *out = NULL);
    /**
     * Delete all buffers and set to initial state.
     */
    void deleteResources();
    void deleteBuffers();
    void deleteQueue();
    void deleteKernels();
    void deleteEvents();
    /**
     * Try to create buffers and set process state to copy buffers.
     * @param error optional string to write error
     * @return success of create kernel buffers
     */
    bool selectDevice(cl_device_type dev_type, cl_uint device_id = 0);
    void printDevicesInfo();

    bool createResources();
    bool createQueue();
    bool createBuffers();
    bool createKernels();
    bool createEvents();

    bool convRgbaGray(cl_mem out_image, cl_mem in_image, cl_uint width, cl_uint height, cl_event *proc_event = NULL);
    bool convGrayRgba(cl_mem out_image, cl_mem in_image, cl_uint width, cl_uint height, cl_event *proc_event = NULL);

    cl_mem d_image_1 = NULL;
    cl_mem d_image_2 = NULL;

    cl_context context = NULL;
    cl_command_queue queue = NULL;
    cl_program program = NULL;
    cl_kernel rgba_gray_kernel = NULL;
    cl_kernel gray_rgba_kernel = NULL;
    cl_device_id device = NULL;
    v_cl_event ev_proc;

    bool valid = false;
    WaveletOpenclParam *param;
  private:
    v_cl_event ev_pre;
    v_cl_event ev_post;
};

#endif
