#ifndef WAVELET_PARAM_H
#define WAVELET_PARAM_H
#include <cstdio>
#include <sstream>
#include "Debug.h"
#include "main.h"


typedef enum e_engine_type
{
    ENGINE_TYPE_OPENMP_SEP = 0,
    ENGINE_TYPE_OPENMP_COMB,
    ENGINE_TYPE_OPENCL_SEP,
    ENGINE_TYPE_OPENCL_COMB    
}engine_type;

/**
 * @brief Wavelet interface
 */
class WaveletParam
{
	public:
		unsigned int max_depth = 1;
		bool interlaced = false;
    /**
    * Transcode frame.
    * @param bgra_frame frame in BGRA format to transcode
    * @param out optional frame process timers
    * @param error optional string to write error
    */
    //void printError(){fprintf(stderr, this->error_msg.str().c_str());}
	protected:
    WaveletParam(engine_type in_engine) : engine(in_engine){};
    ~WaveletParam(){/*this->printError();*/};
    void printDebug()
      {
        std::string engine_type;
        switch(this->engine)
          {
            case ENGINE_TYPE_OPENCL_SEP:
              engine_type = "opencl";
            break;
            case ENGINE_TYPE_OPENCL_COMB:
              engine_type = "openclcomb";
            break;
            case ENGINE_TYPE_OPENMP_SEP:
              engine_type = "openmp";
            break;
          }
        fprintf(stderr, "\nProgram configuration:\n"
                          " engine type:             %s\n", engine_type.c_str());
      }
    engine_type engine;
    //std::stringstream error_msg;
};

#endif

